import java.util.Random;

public class OurLinkedList implements Structure {
    public class Link {
        int value;
        Link next;
        Link previous;

        public Link(Integer value, Link next, Link previous) {
            this.value = value;
            this.next = next;
            this.previous = previous;
        }
        public Link(Integer value) {
        	this.value = value;
            next = this;
            previous = this;
        }
    }

    public class Cursor {
        OurLinkedList list;
        Link link;
        int pos;

        public Cursor(OurLinkedList list, Link link) {
            this.link = link;
            this.list = list;
            pos = 0;
        }

        public void goFor() {
            if (pos == list.nbVal - 1)
                pos = 0;
            else
                pos++;
            link = link.next;
        }

        public void goBack() {
            if (pos == 0)
                pos = list.nbVal  - 1;
            else
                pos--;
            link = link.previous;
        }

        public void setPos(int pos) {
            while(this.pos < pos)
                goFor();
        }
    }

    Link sentinel;
    int nbVal;
    Cursor cursor;
    Random r = new Random();

    public OurLinkedList() {
        sentinel = new Link(0);
        cursor = new Cursor(this, sentinel.previous);
        nbVal = 0;
    }

    public boolean isEmpty() { return nbVal == 0;}

    @Override
    public int get(int pos) {
        if(pos < 0 || pos >= nbVal|| isEmpty()) {
            throw new IndexOutOfBoundsException("elem : mauvaise position du curseur");
        }
        int depart = cursor.pos;
        cursor.setPos(pos);
        int res = cursor.link.value;
        cursor.setPos(depart);
        return res;
    }

    private void insert(int pos, int val) {
        if(pos < 0 || pos > nbVal) {
            throw new IndexOutOfBoundsException("elem : mauvaise position du curseur");
        }
        int depart = cursor.pos;
        cursor.setPos(pos);
        Link link = new Link(val, cursor.link.next, cursor.link);
        cursor.link.next.previous = link;
        cursor.link.next = link;
        cursor.setPos(depart);
    }

    @Override
    public void insertQueue(int elem) {
    	if(nbVal != 0) {
    		insert(nbVal - 1, elem);
    	} else {
    		insert(0, elem);
    	}
        nbVal++;
    }

    @Override
    public void insertRandom(int elem) {
    	if(nbVal != 0) {
    		insert(r.nextInt(nbVal), elem);
    	} else {
    		insert(0, elem);
    	}
        nbVal++;
    }

    @Override
    public void insertHead(int elem) {
        insert(0, elem);
        nbVal++;
    }

    @Override
    public void remove(int pos) {
        if(pos < 0 || pos > nbVal || isEmpty()) {
            throw new IndexOutOfBoundsException("elem : mauvaise position du curseur");
        }
        int depart = cursor.pos;
        cursor.setPos(pos);
        cursor.link.previous.next = cursor.link.next;
        cursor.link.next.previous = cursor.link.previous;
        cursor.setPos(depart);
        
        nbVal--;
    }
}
